local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local print = print

c.Init(a)

a.Costs = {}
local function setCost(blood, frost, unholy, death, name, bonusRP, rime)
	a.Costs[s.SpellName(c.GetID(name))] = {
		Blood = blood,
		Frost = frost,
		Unholy = unholy,
		Death = death,
		BonusRP = bonusRP,
		Rime = rime,
	}
end

setCost(0, 0, 0, 0, "Empower Rune Weapon", 25)
setCost(0, 0, 0, 0, "Horn of Winter", 10)
--setCost(0, 0, 1, 0, "Bone Shield")
--setCost(0, 0, 1, 0, "Dark Transformation")
--setCost(0, 0, 1, 0, "Death and Decay")
setCost(0, 0, 1, 0, "Plague Strike")
--setCost(0, 0, 1, 0, "Scourge Strike")
--setCost(0, 1, 0, 0, "Chains of Ice")
setCost(0, 1, 0, 0, "Howling Blast", nil, true)
setCost(0, 1, 0, 0, "Icy Touch", nil, true)
setCost(0, 1, 0, 0, "Pillar of Frost")
--setCost(0, 1, 1, 0, "Death Strike")
setCost(0, 1, 1, 0, "Obliterate")
--setCost(1, 0, 0, 0, "Blood Boil")
--setCost(1, 0, 0, 0, "Heart Strike")
--setCost(1, 0, 0, 0, "Pestilence")
--setCost(1, 1, 0, 0, "Festering Strike")
setCost(0, 0, 0, 1, "Death Siphon")

local function runeAvailable(runeId, delay)
	if delay == nil then
		delay = 0
	end
	return a.Runes[runeId] <= delay
end

local function fakeConsume(runeID, delay, needed, deathAvailable)
	if runeAvailable(runeID, delay) then
		if a.IsDeathRune(runeID) then
			deathAvailable = deathAvailable + 1
		elseif needed > 0 then
			needed = needed - 1
		end
	end
	return needed, deathAvailable
end

local function masterOverride(z)
	if z.CheckFirst and not z:CheckFirst() then
		return
	end
	
	local name = s.SpellName(z.ID)
	local cost = a.Costs[name]
	if cost then
		local delay = 0
		if z.NoGCD then
			delay = -c.GetBusyTime()
		end
		local deathAvailable = a.PendingDeathRunes
		local bloodNeeded = cost.Blood
		local frostNeeded = cost.Rime and a.FreezingFog > 0 and 0 or cost.Frost
		local unholyNeeded = cost.Unholy
		bloodNeeded, deathAvailable = fakeConsume(
			1, delay, bloodNeeded, deathAvailable)
		bloodNeeded, deathAvailable = fakeConsume(
			2, delay, bloodNeeded, deathAvailable)
		frostNeeded, deathAvailable = fakeConsume(
			5, delay, frostNeeded, deathAvailable)
		frostNeeded, deathAvailable = fakeConsume(
			6, delay, frostNeeded, deathAvailable)
		unholyNeeded, deathAvailable = fakeConsume(
			3, delay, unholyNeeded, deathAvailable)
		unholyNeeded, deathAvailable = fakeConsume(
			4, delay, unholyNeeded, deathAvailable)
	if bloodNeeded + frostNeeded + unholyNeeded + cost.Death
		> deathAvailable then
		
		return false
	end
end
--c.Debug("Resources", s.SpellCost(name), "<=", a.RP, "|", s.SpellCost(name) <= a.RP)
	return s.SpellCost(name) <= a.RP
end

-- Use this for all spells with rune or RP costs.  Spells added this way may
-- only use the CheckFirst attribute.  All others will be ignored.  Cooldowns
-- will NOT be checked.
local function addSpell(name, tag, attributes)
	c.AddSpell(name, tag, attributes).Override = masterOverride
end

local function addOptionalSpell(name, tag, attributes)
	c.AddOptionalSpell(name, tag, attributes).Override = masterOverride
end

------------------------------------------------------------------------ common
local function isFullyDepleted(runeID)
	local otherID
	if runeID % 2 == 0 then
		otherID = runeID - 1
	else
		otherID = runeID + 1
	end
	local otherCD = a.Runes[otherID]
	return otherCD > 0 and a.Runes[runeID] > otherCD
end

local function hasFullyDepletedRune()
	for runeID = 1, 6 do
		if isFullyDepleted(runeID) then
			return true
		end
	end
end

function a.BothRunesAvailable(runeName, delay)
	if runeName == "Blood" then
		return runeAvailable(1, delay) and runeAvailable(2, delay)
	elseif runeName == "Frost" then
		return runeAvailable(5, delay) and runeAvailable(6, delay)
	elseif runeName == "Unholy" then
		return runeAvailable(3, delay) and runeAvailable(4, delay)
	end
	print("Bad rune name: ", runeName)
end

local function hasFrostFever(earlyRefresh)
	return c.GetMyDebuffDuration("Frost Fever") > earlyRefresh
		or c.IsAuraPendingFor("Outbreak")
		or c.IsAuraPendingFor("Icy Touch")
		or c.IsAuraPendingFor("Howling Blast")
end

local function hasBloodPlague(earlyRefresh)
	return c.GetMyDebuffDuration("Blood Plague") > earlyRefresh
		or c.IsAuraPendingFor("Outbreak")
		or c.IsAuraPendingFor("Plague Strike")
end

local function hasBothDiseases()
	return hasFrostFever(0) and hasBloodPlague(0)
end

c.AddSpell("Horn of Winter", nil, {
	CheckFirst = function()
		return s.PowerMissing("player") > 10
	end
})

c.AddOptionalSpell("Empower Rune Weapon", "when low", {
	NoGCD = true,
	CheckFirst = function()
		return not runeAvailable(1)
			and not runeAvailable(2)
			and not runeAvailable(3)
			and not runeAvailable(4)
			and not runeAvailable(5)
			and not runeAvailable(6)
			and s.Power("player") < 32
	end
})

c.AddSpell("Outbreak", nil, {
	CheckFirst = function()
		return not hasBothDiseases()
	end
})
c.RegisterAura("Outbreak", "Frost Fever")
c.RegisterAura("Icy Touch", "Frost Fever")
c.RegisterAura("Plague Strike", "Blood Plague")

c.AddOptionalSpell("Blood Tap", nil, {
	NoGCD = true,
	CheckFirst = function()
		local stack = c.GetBuffStack("Blood Charge")
		if stack < 5 then
			return false
		end
		
		if stack >= 10 then
			return hasFullyDepletedRune()
		end
		
		return isFullyDepleted(3)
			or isFullyDepleted(4)
			or isFullyDepleted(5)
			or isFullyDepleted(6)
	end
})

c.AddInterrupt("Mind Freeze", nil, {
	NoGCD = true,
})

------------------------------------------------------------------------- frost
c.AddOptionalSpell("Frost Presence", nil, { Type = "form" })

addSpell("Frost Strike")

addSpell("Frost Strike", "under KM", {
	CheckFirst = function()
		return a.KillingMachine
	end
})

addSpell("Frost Strike", "under KM no Diseases", {
	CheckFirst = function()
		return a.KillingMachine and not hasBothDiseases()
	end
})

addSpell("Frost Strike", "for Resources", {
	CheckFirst = function()
		if c.HasTalent("Blood Tap") then
			return c.GetBuffStack("Blood Charge") <= 10
		elseif c.HasTalent("Runic Empowerment") then
			return hasFullyDepletedRune()
		elseif c.HasTalent("Runic Corruption") then
			return c.GetBuffDuration("Runic Corruption") < 1
		end
	end
})

c.RegisterAura("Howling Blast", "Frost Fever")

addSpell("Howling Blast")

addSpell("Howling Blast", "for FF", {
	CheckFirst = function()
		return not hasFrostFever(0)
	end
})

addSpell("Plague Strike", "for BP unless Outbreak", {
	CheckFirst = function()
		return not hasBloodPlague(0)
			and (not s.Flashable(c.GetID("Outbreak"))
				or c.GetCooldown("Outbreak"))
	end
})

addSpell("Obliterate", nil, {
	CheckFirst = function()
		return hasBothDiseases()
	end
})

addSpell("Obliterate", "UU", {
	CheckFirst = function()
		return a.BothRunesAvailable("Unholy", 1) and hasBothDiseases()
	end
})

addSpell("Obliterate", "under KM no Freezing Fog", {
	CheckFirst = function()
		return a.KillingMachine
			and a.FreezingFog == 0
			and hasBothDiseases()
	end
})

c.AddSpell("Plague Leech", nil, {
	CheckFirst = function()
		if not hasBothDiseases() or not hasFullyDepletedRune() then
			return false
		end
		
		if c.GetCooldown("Outbreak") <= 1 and not c.IsCasting("Outbreak") then
			return true
		end
		
		if not c.HasBuff("Freezing Fog") or c.IsCasting("Howling Blast") then
			return false
		end
		
		for runeID = 3, 6 do
			if runeAvailable(runeID) then
				return true
			end
		end
		return false
	end
})

addSpell("Death Siphon", nil, {
	Run = function()
		c.Debug("Death Siphon", "running")
	end
})

addSpell("Death Siphon", "DD", {
	CheckFirst = function()
		return a.BothRunesAvailable("Blood", 1)
	end
})

c.AddOptionalSpell("Pillar of Frost", nil, { 
	NoGCD = true,
})

addSpell("Howling Blast", "BB or FF", {
	CheckFirst = function()
		return a.BothRunesAvailable("Blood", 1)
			or a.BothRunesAvailable("Frost", 1)
	end
})

c.CloneSpell("Death and Decay", "UU", {
	CheckFirst = function()
		return a.BothRunesAvailable("Unholy", 1)
	end
})

addSpell("Plague Strike", "UU", {
	CheckFirst = function()
		return a.BothRunesAvailable("Unholy", 1)
	end
})
